@@ -20,6 +20,8 @@
 
 #include "Poco/Net/Net.h"
 #include "Poco/Net/Socket.h"
+#include "Poco/Net/SocketNotification.h"
+#include "Poco/Net/SocketNotifier.h"
 #include "Poco/Net/PollSet.h"
 #include "Poco/Runnable.h"
 #include "Poco/Timespan.h"
@@ -40,8 +42,6 @@ namespace Net {
 
 
 class Socket;
-class SocketNotification;
-class SocketNotifier;
 
 
 class Net_API SocketReactor: public Poco::Runnable
@@ -72,31 +72,34 @@ class Net_API SocketReactor: public Poco::Runnable
 	/// as argument.
 	///
 	/// Once started, the SocketReactor waits for events
-	/// on the registered sockets, using Socket::select().
+	/// on the registered sockets, using PollSet:poll().
 	/// If an event is detected, the corresponding event handler
 	/// is invoked. There are five event types (and corresponding
 	/// notification classes) defined: ReadableNotification, WritableNotification,
-	/// ErrorNotification, TimeoutNotification, IdleNotification and
-	/// ShutdownNotification.
+	/// ErrorNotification, TimeoutNotification and ShutdownNotification.
 	///
 	/// The ReadableNotification will be dispatched if a socket becomes
 	/// readable. The WritableNotification will be dispatched if a socket
 	/// becomes writable. The ErrorNotification will be dispatched if
 	/// there is an error condition on a socket.
 	///
-	/// If the timeout expires and no event has occurred, a
+	/// Timeout/sleep strategy operates as follows:
+	///
+	/// If the poll timeout expires and no event has occurred, a
 	/// TimeoutNotification will be dispatched to all event handlers
 	/// registered for it. This is done in the onTimeout() method
 	/// which can be overridden by subclasses to perform custom
 	/// timeout processing.
 	///
-	/// If there are no sockets for the SocketReactor to pass to
-	/// Socket::select(), an IdleNotification will be dispatched to
-	/// all event handlers registered for it. This is done in the
-	/// onIdle() method which can be overridden by subclasses
-	/// to perform custom idle processing. Since onIdle() will be
-	/// called repeatedly in a loop, it is recommended to do a
-	/// short sleep or yield in the event handler.
+	/// By default, the SocketReactor is configured to start sleeping
+	/// when the poll timeout is zero and there are no socket events for
+	/// a certain amount of time; sleep duration is progressive, up to
+	/// the configured limit. This behavior can be disabled through
+	/// configuration parameters.
+	///
+	/// When there are no registered handlers, the SocketRactor sleeps
+	/// an incremental amount of milliseconds, up to the sleep limit.
+	/// Increment step value and sleep limit are configurable.
 	///
 	/// Finally, when the SocketReactor is about to shut down (as a result
 	/// of stop() being called), it dispatches a ShutdownNotification
@@ -104,28 +107,84 @@ class Net_API SocketReactor: public Poco::Runnable
 	/// which can be overridden by subclasses to perform custom
 	/// shutdown processing.
 	///
-	/// The SocketReactor is designed to run in its own thread. It is
-	/// possible to run multiple SocketReactors in parallel, as long as
-	/// they work on different sockets.
+	/// The SocketReactor is implemented so that it can run in its own thread.
+	/// Moreover, the thread affinity to a CPU core can optionally be set for the
+	/// thread on platforms where that functionality is supported and implemented.
+	/// It is also possible to run multiple SocketReactors in parallel, as long
+	/// as they work on different sockets.
 	///
-	/// It is safe to call addEventHandler() and removeEventHandler()
-	/// from another thread while the SocketReactor is running. Also,
-	/// it is safe to call addEventHandler() and removeEventHandler()
-	/// from event handlers.
+	/// It is safe to call addEventHandler() and removeEventHandler() from another
+	/// thread while the SocketReactor is running. Also, it is safe to call
+	/// addEventHandler() and removeEventHandler() from event handlers.
+	///
+	/// SocketReactor uses NotificationCenter to notify observers. When a handler
+	/// throws an exception, the NotificationCenter stops notifying the rest of
+	/// the observers about that particular event instance and propagates the
+	/// exception, which is eventually caught in the SocketReactor::run() method.
+	/// This sequence of events is obviously not desirable and it is highly
+	/// recommended that handlers wrap the code in try/catch and deal with all
+	/// the exceptions internally, lest they disrupt the notification of the peers.
 {
 public:
+	struct Params
+		/// Reactor parameters.
+		/// Default values should work well for most scenarios.
+		///
+		/// Note: the default behavior on zero poll timeout is to start
+		/// incrementally sleeping after `idleThreshold` and no socket events.
+		/// This prevents high CPU usage during periods without network
+		/// activity. To disable it, set `throttle` to false.
+	{
+		Poco::Timespan pollTimeout = DEFAULT_TIMEOUT;
+			/// Timeout for PolllSet::poll()
+
+		long sleep = 0;
+			/// Amount of milliseconds to sleep, progressively incremented,
+			/// at `increment` step, up to the `sleepLimit`.
+
+		long sleepLimit = DEFAULT_SLEEP_LIMIT;
+			/// Max sleep duration in milliseconds
+			/// This is the ceiling value in milliseconds for the sleep algorithm,
+			/// which kicks in in two cases:
+			///
+			///   - when there are no subscribers and the reactor is just idle-spinning
+			///   - when there are subscribers, but there was no socket events signalled
+			///     for `sleepLimit` milliseconds and `throttle` is true
+
+		int increment = 1;
+			/// Increment value for the sleep backoff algorithm.
+
+		long idleThreshold = DEFAULT_SLEEP_LIMIT;
+			/// Indicates when to start sleeping (throttling) on zero poll timeout
+
+		bool throttle = true;
+			/// Indicates whether to start sleeping when poll timeout is zero and
+			/// there's no socket events for a period longer than `idleThreshold`
+	};
+
 	SocketReactor();
 		/// Creates the SocketReactor.
 
-	explicit SocketReactor(const Poco::Timespan& timeout);
-		/// Creates the SocketReactor, using the given timeout.
+	explicit SocketReactor(const Poco::Timespan& pollTimeout, int threadAffinity = -1);
+		/// Creates the SocketReactor, using the given poll timeout.
+		///
+		/// The threadAffinity argument, when non-negative, indicates on which CPU core should
+		/// the run() method run. Nonexisting core or situation when this feature is not implemented
+		/// are silently ignored and this argument has no effect in such scenarios.
+
+	SocketReactor(const Params& params, int threadAffinity = -1);
+		/// Creates the SocketReactor, using the given parameters.
+		/// The threadAffinity argument, when non-negative, indicates on which CPU core should
+		/// the run() method run. Nonexisting core or situation when this feature is not implemented
+		/// are silently ignored and this argument has no effect in such scenarios.
 
 	virtual ~SocketReactor();
 		/// Destroys the SocketReactor.
 
-	void run();
+	virtual void run();
 		/// Runs the SocketReactor. The reactor will run
 		/// until stop() is called (in a separate thread).
+		/// Can be overriden by inheriting classes.
 
 	void stop();
 		/// Stops the SocketReactor.
@@ -139,13 +198,12 @@ public:
 	void setTimeout(const Poco::Timespan& timeout);
 		/// Sets the timeout.
 		///
-		/// If no other event occurs for the given timeout
+		/// If no socket event occurs for the given timeout
 		/// interval, a timeout event is sent to all event listeners.
 		///
 		/// The default timeout is 250 milliseconds;
 		///
-		/// The timeout is passed to the Socket::select()
-		/// method.
+		/// The timeout is passed to the PollSet::poll() method.
 
 	const Poco::Timespan& getTimeout() const;
 		/// Returns the timeout.
@@ -178,13 +236,6 @@ protected:
 		/// dispatches the TimeoutNotification and thus should be called by overriding
 		/// implementations.
 
-	virtual void onIdle();
-		/// Called if no sockets are available to call select() on.
-		///
-		/// Can be overridden by subclasses. The default implementation
-		/// dispatches the IdleNotification and thus should be called by overriding
-		/// implementations.
-
 	virtual void onShutdown();
 		/// Called when the SocketReactor is about to terminate.
 		///
@@ -192,12 +243,11 @@ protected:
 		/// dispatches the ShutdownNotification and thus should be called by overriding
 		/// implementations.
 
-	virtual void onBusy();
-		/// Called when the SocketReactor is busy and at least one notification
-		/// has been dispatched.
-		///
-		/// Can be overridden by subclasses to perform additional
-		/// periodic tasks. The default implementation does nothing.
+	void onError(const Socket& socket, int code, const std::string& description);
+		/// Notifies all subscribers when the reactor loop throws an exception.
+
+	void onError(int code, const std::string& description);
+		/// Notifies all subscribers when the reactor loop throws an exception.
 
 	void dispatch(const Socket& socket, SocketNotification* pNotification);
 		/// Dispatches the given notification to all observers
@@ -207,30 +257,35 @@ protected:
 		/// Dispatches the given notification to all observers.
 
 private:
-	typedef Poco::AutoPtr<SocketNotifier>     NotifierPtr;
-	typedef Poco::AutoPtr<SocketNotification> NotificationPtr;
-	typedef std::map<poco_socket_t, NotifierPtr>     EventHandlerMap;
-	typedef Poco::FastMutex                   MutexType;
-	typedef MutexType::ScopedLock             ScopedLock;
+	typedef Poco::AutoPtr<SocketNotifier>        NotifierPtr;
+	typedef Poco::AutoPtr<SocketNotification>    NotificationPtr;
+	typedef std::map<poco_socket_t, NotifierPtr> EventHandlerMap;
+	typedef Poco::FastMutex                      MutexType;
+	typedef MutexType::ScopedLock                ScopedLock;
 
 	bool hasSocketHandlers();
 	void dispatch(NotifierPtr& pNotifier, SocketNotification* pNotification);
 	NotifierPtr getNotifier(const Socket& socket, bool makeNew = false);
 
+	void sleep();
+
 	enum
 	{
-		DEFAULT_TIMEOUT = 250000
+		DEFAULT_TIMEOUT = 250000,
+			/// Default timeout for PollSet::poll()
+		DEFAULT_SLEEP_LIMIT = 250
+			/// Default limit for event-based sleeping
 	};
 
+	Params            _params;
+	int               _threadAffinity = -1;
 	std::atomic<bool> _stop;
-	Poco::Timespan    _timeout;
 	EventHandlerMap   _handlers;
 	PollSet           _pollSet;
 	NotificationPtr   _pReadableNotification;
 	NotificationPtr   _pWritableNotification;
 	NotificationPtr   _pErrorNotification;
 	NotificationPtr   _pTimeoutNotification;
-	NotificationPtr   _pIdleNotification;
 	NotificationPtr   _pShutdownNotification;
 	MutexType         _mutex;
 	Poco::Event       _event;
@@ -238,6 +293,46 @@ private:
 	friend class SocketNotifier;
 };
 
+//
+// inlines
+//
+
+
+inline void SocketReactor::setTimeout(const Poco::Timespan& timeout)
+{
+	_params.pollTimeout = timeout;
+}
+
+
+inline const Poco::Timespan& SocketReactor::getTimeout() const
+{
+	return _params.pollTimeout;
+}
+
+
+inline bool SocketReactor::has(const Socket& socket) const
+{
+	return _pollSet.has(socket);
+}
+
+
+inline void SocketReactor::onError(const Socket& socket, int code, const std::string& description)
+{
+	dispatch(new ErrorNotification(this, socket, code, description));
+}
+
+
+inline void SocketReactor::onError(int code, const std::string& description)
+{
+	dispatch(new ErrorNotification(this, code, description));
+}
+
+
+inline void SocketReactor::dispatch(NotifierPtr& pNotifier, SocketNotification* pNotification)
+{
+	pNotifier->dispatch(pNotification);
+}
+
 
 } } // namespace Poco::Net
 
